<?php

/**
 * @package     Extly.Components
 * @subpackage  com_autotweet - AutoTweetNG posts content to social channels (Scoopit, Facebook, LinkedIn, etc).
 *
 * @author      Prieco S.A. <support@extly.com>
 * @copyright   Copyright (C) 2007 - 2015 Prieco, S.A. All rights reserved.
 * @license     http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        http://www.extly.com http://support.extly.com
 */
// No direct access
defined('_JEXEC') or die('Restricted access');

/**
 * AutoTweet JomSocial channel.
**/

JLoader::import('channel', dirname(__FILE__));

include_once JPATH_ROOT . '/components/com_community/libraries/core.php';

/**
 * JomSocialChannelHelper
 *
 * @package     Extly.Components
 * @subpackage  com_autotweet
 * @since       1.0
 */
class JomSocialChannelHelper extends ChannelHelper
{
	/**
	 * sendMessage
	 *
	 * @param   string  $message  Param
	 * @param   array   $data     Param
	 *
	 * @return	array
	 */
	public function sendMessage($message, $data)
	{
		try
		{
			$media_mode = $this->getMediaMode();

			$image_url = null;

			if ($media_mode != 'message')
			{
				$image_url = $data->image_url;
			}

			if (!class_exists('CFactory'))
			{
				throw new Exception('JomSocial is not installed!');
			}

			// JomSocial - ajaxStreamAdd

			$cache = CFactory::getFastCache();
			$cache->clean(array('activities'));

			$author = $data->xtform->get('author');
			$userId = JUserHelper::getUserId($author);
			$my = CFactory::getUser($userId);
			$userparams = $my->getParams();

			// IsRegisteredUser
			if (! (($my->id != 0) && ($my->block != 1)))
			{
				throw new Exception('Block Unregister');
			}

			// @rule: In case someone bypasses the status in the html, we enforce the character limit.
			$config = CFactory::getConfig();

			if (JString::strlen($message) > $config->get('statusmaxchar'))
			{
				$message = JHTML::_('string.truncate', $message, $config->get('statusmaxchar'));
			}

			$message = JString::trim($message);
			$rawMessage = $message;

			/*
			 @rule: Autolink hyperlinks
			@rule: Autolink to users profile when message contains @username
			$message		= CUserHelper::replaceAliasURL($message); // the processing is done on display side
			*/
			$emailMessage = CUserHelper::replaceAliasURL($rawMessage, true);

			// @rule: Spam checks
			if ($config->get('antispam_akismet_status'))
			{
				$filter = CSpamFilter::getFilter();
				$filter->setAuthor($my->getDisplayName());
				$filter->setMessage($message);
				$filter->setEmail($my->email);
				$filter->setURL(CRoute::_('index.php?option=com_community&view=profile&userid=' . $my->id));
				$filter->setType('message');
				$filter->setIP($_SERVER['REMOTE_ADDR']);

				if ($filter->isSpam())
				{
					throw new Exception('Spam');
				}
			}

			$attachment = array();
			$attachment['element'] = 'profile';
			$attachment['target'] = $my->id;
			$attachment['privacy'] = PRIVACY_PUBLIC;

			// Message - Profile case
			$attachment['type'] = 'message';

			// If (COwnerHelper::isMine($my->id, $attachment['target']))
			// Save the message
			$status = CFactory::getModel('status');

			/* If no privacy in attachment than we apply default: Public */
			if (!isset($attachment['privacy']))
			{
				$attachment['privacy'] = COMMUNITY_STATUS_PRIVACY_PUBLIC;
			}

			$status->update($my->id, $rawMessage, $attachment['privacy']);

			// Set user status for current session.
			$today = JFactory::getDate();
			$message2 = (empty($message)) ? ' ' : $message;
			$my->set('_status', $rawMessage);
			$my->set('_posted_on', $today->toSql());

			// Order of replacement
			$order = array("\r\n", "\n", "\r");
			$replace = '<br />';

			// Processes \r\n's first so they aren't converted twice.
			$messageDisplay = str_replace($order, $replace, $message);
			$messageDisplay = CKses::kses($messageDisplay, CKses::allowed());

			// Case 'profile':
			// Push to activity stream
			$act = new StdClass;
			$act->cmd = 'profile.status.update';
			$act->actor = $my->id;
			$act->target = $attachment['target'];
			$act->title = $message;
			$act->content = '';
			$act->app = $attachment['element'];
			$act->cid = $my->id;
			$act->access = $attachment['privacy'];
			$act->comment_id = CActivities::COMMENT_SELF;
			$act->comment_type = 'profile.status';
			$act->like_id = CActivities::LIKE_SELF;
			$act->like_type = 'profile.status';

			$activityParams = new CParameter('');
			$activityParams->set('joocial_message', $message);
			$activityParams->set('joocial_title', $data->title);
			$activityParams->set('joocial_url', $data->url);

			if ($image_url)
			{
				$activityParams->set('joocial_image_url', $data->image_url);
				$media = array(
					'type' => 'image',
					'title' => $data->title,
					'description' => $message,
					'image' => $image_url
				);
				$activityParams->set('headMetas', json_encode($media));
			}

			$act->params = $activityParams->toString();

			/* Let use our new CApiStream */
			$activityData = CApiActivities::add($act);

			CTags::add($activityData);
			CUserPoints::assignPoint('profile.status.update');

			$recipient = CFactory::getUser($attachment['target']);
			$params = new CParameter('');
			$params->set('actorName', $my->getDisplayName());
			$params->set('recipientName', $recipient->getDisplayName());
			$url = CUrlHelper::userLink($act->target, false);
			$params->set('url', $url);
			$params->set('message', $message);

			CNotificationLibrary::add('profile_status_update', $my->id, $attachment['target'], JText::sprintf('COM_COMMUNITY_FRIEND_WALL_POST', $my->getDisplayName()), '', 'wall.post', $params);

			// Email and add notification if user are tagged
			CUserHelper::parseTaggedUserNotification($message, $my, $activityData, array('type' => 'post-comment'));

			$url = RouteHelp::getInstance()->getAbsoluteUrl($url);
			$result = array(
							true,
							'OK - ' . $url
			);
		}
		catch (Exception $e)
		{
			return array(
				false,
				$e->getMessage()
			);
		}

		return $result;
	}

	/**
	 * includeHashTags
	 *
	 * @return  bool
	 */
	public function includeHashTags()
	{
		return $this->channel->params->get('hashtags', true);
	}

	/**
	 * getMaxChars
	 *
	 * @return  int
	 */
	public function getMaxChars()
	{
		if (!class_exists('CFactory'))
		{
			throw new Exception('JomSocial is not installed!');
		}

		$config = CFactory::getConfig();

		return $config->get('statusmaxchar');
	}
}
