<?php

/**
 * @package     Extly.Components
 * @subpackage  com_autotweet - AutoTweetNG posts content to social channels (Scoopit, Facebook, LinkedIn, etc).
 *
 * @author      Prieco S.A. <support@extly.com>
 * @copyright   Copyright (C) 2007 - 2015 Prieco, S.A. All rights reserved.
 * @license     http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        http://www.extly.com http://support.extly.com
 */
// No direct access
defined('_JEXEC') or die('Restricted access');

/**
 * AutoTweet Scoopit channel.
**/

JLoader::import('channel', dirname(__FILE__));
require_once dirname(__FILE__) . '/OAuth/OAuth.php';
JLoader::register('ScoopIt', dirname(__FILE__) . '/Scoopit/ScoopIt.php');
require dirname(__FILE__) . '/Scoopit/oauth/tokenStore/TokenStore.php';

/**
 * ScoopitChannelHelper
 *
 * @package     Extly.Components
 * @subpackage  com_autotweet
 * @since       1.0
 */
class ScoopitChannelHelper extends ChannelHelper
{
	protected $scoopitClient = null;

	protected $tokenStore = null;

	protected $consumer_key = null;

	protected $consumer_secret = null;

	protected $access_token = null;

	protected $is_auth = null;

	protected $me = null;

	/**
	 * ChannelHelper.
	 *
	 * @param   object  $channel  Params.
	 */
	public function __construct($channel)
	{
		parent::__construct($channel);

		if ($channel->id)
		{
			$this->consumer_key = $this->channel->params->get('consumer_key');
			$this->consumer_secret = $this->channel->params->get('consumer_secret');

			$this->tokenStore = new ScoopItSessionTokenStore;

			$access_token = $this->channel->params->get('access_token');
			$access_secret = $this->channel->params->get('access_secret');
			$this->setAccessToken($access_token);
			$this->setAccessTokenSecret($access_secret);
		}
	}

	/**
	 * setAccessToken
	 *
	 * @param   string  $access_token  Param
	 *
	 * @return	void
	 */
	public function setAccessToken($access_token)
	{
		$this->tokenStore->storeAccessToken($access_token);
	}

	/**
	 * setAccessTokenSecret
	 *
	 * @param   string  $access_secret  Param
	 *
	 * @return	void
	 */
	public function setAccessTokenSecret($access_secret)
	{
		$this->tokenStore->storeSecret($access_secret);
	}

	/**
	 * getAccessToken
	 *
	 * @return	string
	 */
	public function getAccessToken()
	{
		return $this->tokenStore->getAccessToken();
	}

	/**
	 * getAccessTokenSecret
	 *
	 * @return	string
	 */
	public function getAccessTokenSecret()
	{
		return $this->tokenStore->getSecret();
	}

	/**
	 * Internal service functions
	 *
	 * @return	object
	 */
	protected function getApiInstance()
	{
		if (!$this->scoopitClient)
		{
			if ((isset($this->channel->id)) && ($this->channel->id))
			{
				require_once dirname(__FILE__) . '/../../controllers/scoopitchannels.php';

				$this->redirectUri = AutotweetControllerScoopitChannels::getCallbackUrl($this->channel->id);
			}

			JLoader::load('ScoopIt');

			$this->scoopitClient = new ScoopitPHP\ScoopIt(
					$this->tokenStore,

					// $myUrl,
					$this->redirectUri,

					// $consumerKey,
					$this->consumer_key,

					// $consumerSecret,
					$this->consumer_secret
			);
		}

		return $this->scoopitClient;
	}

	/**
	 * isAuth()
	 *
	 * @return	mixed
	 */
	public function isAuth()
	{
		$ch = F0FTable::getAnInstance('Channel', 'AutoTweetTable');
		$access_token = $this->getAccessToken();

		if (empty($access_token))
		{
			$this->access_token = null;

			return false;
		}

		try
		{
			$this->getApiInstance();
			$test = $this->scoopitClient->test();

			return $test;
		}
		catch (Exception $e)
		{
			$logger = AutotweetLogger::getInstance();
			$logger->log(JLog::ERROR, $e->getMessage());

			// Invalidating access_token
			$ch->setToken($this->channel->id, 'access_token', '');
		}

		return false;
	}

	/**
	 * profile()
	 *
	 * @return	object
	 */
	public function profile()
	{
		try
		{
			$this->getApiInstance();
			$profile = $this->scoopitClient->profile(null);

			return $profile;
		}
		catch (Exception $e)
		{
			$logger = AutotweetLogger::getInstance();
			$logger->log(JLog::ERROR, $e->getMessage());
		}

		return false;
	}

	/**
	 * authenticate
	 *
	 * @param   string  $oauth_token     Param
	 * @param   string  $oauth_verifier  Param
	 *
	 * @return	bool
	 */
	public function authenticate($oauth_token, $oauth_verifier)
	{
		$this->getApiInstance();

		$url = $this->scoopitClient->login();

		$ch = F0FTable::getAnInstance('Channel', 'AutoTweetTable');

		// No redirection
		if (!$url)
		{
			$ch->setToken($this->channel->id, 'access_token', $this->getAccessToken());
			$ch->setToken($this->channel->id, 'access_secret', $this->getAccessTokenSecret());

			return true;
		}
		else
		{
			JFactory::getApplication()->enqueueMessage('Unable to retrieve access token.', 'error');

			// Invalidating access_token
			$ch->setToken($this->channel->id, 'access_token', '');
		}

		return false;
	}

	/**
	 * getAuthorizationUrl
	 *
	 * @return	string
	 */
	public function getAuthorizationUrl()
	{
		$this->getApiInstance();

		return $this->scoopitClient->login();
	}

	/**
	 * getExpiresIn
	 *
	 * @return	string
	 */
	public function getExpiresIn()
	{
		if (($this->access_token) && (!empty($this->access_token)))
		{
			$access_token = json_decode($this->access_token);

			$created = $access_token->created;
			$expires_in = $access_token->expires_in;

			$expires_in += $created;

			return JHtml::_('date', $expires_in, JText::_('COM_AUTOTWEET_DATE_FORMAT'));
		}

		return null;
	}

	/**
	 * sendMessage
	 *
	 * @param   string  $message  Param
	 * @param   array   $data     Param
	 *
	 * @return	array
	 */
	public function sendMessage($message, $data)
	{
		$isAuth = $this->isAuth();

		if (!$isAuth)
		{
			return array(
				false,
				JText::_('COM_AUTOTWEET_CHANNEL_SCOOPIT_NOT_AUTH_ERR')
			);
		}

		$result = array(false, 'Scoopit Unknown', null);

		try
		{
			$media_mode = $this->getMediaMode();
			$image_url = null;

			if ($media_mode != 'message')
			{
				$image_url = $data->image_url;
			}

			$topic_id = $this->channel->params->get('topic_id');

			$this->getApiInstance();
			$response = $this->scoopitClient->createAPost(
					$data->title,
					$data->url,
					$message,
					$image_url,
					$topic_id
			);

			$result = array(
				$response->success,
				$response->post->id . ' - ' . $response->post->scoopShortUrl
			);
		}
		catch (Exception $e)
		{
			return array(
				false,
				$e->getMessage()
			);
		}

		return $result;
	}

	/**
	 * includeHashTags
	 *
	 * @return  bool
	 */
	public function includeHashTags()
	{
		return $this->channel->params->get('hashtags', true);
	}

	/**
	 * getSocialUrl
	 *
	 * @param   string  $userId  Param
	 *
	 * @return	string
	 */
	public function getSocialUrl($userId)
	{
		return 'http://www.scoop.it/u/' . $userId;
	}

	/**
	 * searchTopics.
	 *
	 * @param   string  $query  Params
	 *
	 * @return	array
	 */
	public function searchTopics($query)
	{
		$result = array();
		$this->getApiInstance();

		$lang = RouteHelp::getInstance()->getLanguageSef();
		$response = $this->scoopitClient->search($query, 'topic', 50, 0, $lang);

		$items_data = array();

		if ( ($response) && (isset($response->topics)) && (!empty($response->topics)) )
		{
			$items_data = $response->topics;
		}

		$result[] = array(
			'type' => 'Topic',
			'id' => 0,
			'name' => JText::_('JSELECT')
		);

		if (!empty($items_data))
		{
			foreach ($items_data as $item)
			{
				$name = $item->name;

				if ($item->isCurator)
				{
					$name .= ' (C)';
				}

				$a = array(
					'type' => 'Topic',
					'id' => $item->id,
					'name' => $name
				);
				$result[] = $a;
			}
		}

		return $result;
	}
}

/**
 * ScoopItSessionTokenStore
 * Store authentication tokens in the session
 *
 * @package     Extly.Components
 * @subpackage  com_autotweet
 * @since       1.0
 */
class ScoopItSessionTokenStore implements ScoopitPHP\TokenStore
{
	protected $register = null;

	/**
	 * ScoopItSessionTokenStore.
	 *
	 */
	public function __construct()
	{
		$this->register = JFactory::getSession();
	}

	/**
	 * storeRequestToken
	 *
	 * @param   string  $value  Param
	 *
	 * @return	void
	 */
	public function storeRequestToken($value)
	{
		$this->register->set('scoop.requestToken', $value);
	}

	/**
	 * storeAccessToken
	 *
	 * @param   string  $value  Param
	 *
	 * @return	void
	 */
	public function storeAccessToken($value)
	{
		$this->register->set('scoop.accessToken', $value);
	}

	/**
	 * storeVerifier
	 *
	 * @param   string  $value  Param
	 *
	 * @return	void
	 */
	public function storeVerifier($value)
	{
		$this->register->set('scoop.verifier', $value);
	}

	/**
	 * storeSecret
	 *
	 * @param   string  $value  Param
	 *
	 * @return	void
	 */
	public function storeSecret($value)
	{
		$this->register->set('scoop.secret', $value);
	}

	/**
	 * getRequestToken
	 *
	 * @return	string
	 */
	public function getRequestToken()
	{
		return $this->register->get('scoop.requestToken');
	}

	/**
	 * getAccessToken
	 *
	 * @return	string
	 */
	public function getAccessToken()
	{
		return $this->register->get('scoop.accessToken');
	}

	/**
	 * getVerifier
	 *
	 * @return	string
	 */
	public function getVerifier()
	{
		return $this->register->get('scoop.verifier');
	}

	/**
	 * getSecret
	 *
	 * @return	string
	 */
	public function getSecret()
	{
		return $this->register->get('scoop.secret');
	}

	/**
	 * flushRequestToken
	 *
	 * @return	string
	 */
	public function flushRequestToken()
	{
		$this->register->set('scoop.requestToken', null);
	}

	/**
	 * flushAccessToken
	 *
	 * @return	string
	 */
	public function flushAccessToken()
	{
		$this->register->set('scoop.accessToken', null);
	}

	/**
	 * flushVerifier
	 *
	 * @return	string
	 */
	public function flushVerifier()
	{
		$this->register->set('scoop.verifier', null);
	}

	/**
	 * flushSecret
	 *
	 * @return	string
	 */
	public function flushSecret()
	{
		$this->register->set('scoop.secret', null);
	}
}
