<?php

/**
 * @package     Extly.Components
 * @subpackage  com_autotweet - AutoTweetNG posts content to social channels (Tumblr, Facebook, LinkedIn, etc).
 *
 * @author      Prieco S.A. <support@extly.com>
 * @copyright   Copyright (C) 2007 - 2015 Prieco, S.A. All rights reserved.
 * @license     http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        http://www.extly.com http://support.extly.com
 */
// No direct access
defined('_JEXEC') or die('Restricted access');

/**
 * AutoTweet Tumblr channel.
**/

JLoader::import('channel', dirname(__FILE__));
require_once dirname(__FILE__) . '/OAuth/OAuth.php';
JLoader::register('TumblrOAuth', dirname(__FILE__) . '/tumblroauth/tumblroauth.php');

/**
 * TumblrChannelHelper
 *
 * @package     Extly.Components
 * @subpackage  com_autotweet
 * @since       1.0
 */
class TumblrChannelHelper extends ChannelHelper
{
	protected $tumblrClient = null;

	protected $consumer_key = null;

	protected $consumer_secret = null;

	protected $access_token = null;

	protected $access_secret = null;

	protected $is_auth = null;

	protected $me = null;

	/**
	 * ChannelHelper.
	 *
	 * @param   object  $channel  Params.
	 */
	public function __construct($channel)
	{
		parent::__construct($channel);

		if ($channel->id)
		{
			$this->consumer_key = $this->channel->params->get('consumer_key');
			$this->consumer_secret = $this->channel->params->get('consumer_secret');
			$this->access_token = $this->channel->params->get('access_token');
			$this->access_secret = $this->channel->params->get('access_secret');
		}
	}

	/**
	 * Internal service functions
	 *
	 * @return	object
	 */
	protected function getApiInstance()
	{
		if (!$this->tumblrClient)
		{
			JLoader::load('TumblrOAuth');
			$this->tumblrClient = new tumblroauth\TumblrOAuth(
					$this->consumer_key,
					$this->consumer_secret,
					$this->access_token,
					$this->access_secret
			);
		}

		return $this->tumblrClient;
	}

	/**
	 * isAuth()
	 *
	 * @return	mixed
	 */
	public function isAuth()
	{
		if (empty($this->access_token))
		{
			$this->access_token = null;

			return false;
		}

		try
		{
			$this->getApiInstance();

			$return = $this->tumblrClient->get('user/info');

			if ((!isset($return->meta->status)) || ($return->meta->status == 401))
			{
				return false;
			}

			return $return;
		}
		catch (Exception $e)
		{
			$logger = AutotweetLogger::getInstance();
			$logger->log(JLog::ERROR, $e->getMessage());

			// Invalidating access_token
			$ch->setToken($this->channel->id, 'access_token', '');
		}

		return false;
	}

	/**
	 * authenticate
	 *
	 * @param   string  $oauth_token     Param
	 * @param   string  $oauth_verifier  Param
	 *
	 * @return	bool
	 */
	public function authenticate($oauth_token, $oauth_verifier)
	{
		$this->getApiInstance();
		$access_token = $this->tumblrClient->getAccessToken($oauth_verifier);

		$ch = F0FTable::getAnInstance('Channel', 'AutoTweetTable');

		if (($access_token) && (is_array($access_token)) && (array_key_exists('oauth_token_secret', $access_token)))
		{
			$ch->setToken($this->channel->id, 'access_token', $access_token['oauth_token']);
			$ch->setToken($this->channel->id, 'access_secret', $access_token['oauth_token_secret']);

			return true;
		}
		else
		{
			JFactory::getApplication()->enqueueMessage('Unable to retrieve access token.', 'error');

			// Invalidating access_token
			$ch->setToken($this->channel->id, 'access_token', '');
			$ch->setToken($this->channel->id, 'access_secret', '');
		}

		return false;
	}

	/**
	 * getAuthorizationUrl
	 *
	 * @return	string
	 */
	public function getAuthorizationUrl()
	{
		require_once dirname(__FILE__) . '/../../controllers/tumblrchannels.php';
		$this->getApiInstance();

		$redirectUri = AutotweetControllerTumblrChannels::getCallbackUrl($this->channel->id);
		$request_token = $this->tumblrClient->getRequestToken($redirectUri);
		$url = $this->tumblrClient->getAuthorizeURL($request_token);

		$ch = F0FTable::getAnInstance('Channel', 'AutoTweetTable');
		$ch->setToken($this->channel->id, 'access_token', $request_token['oauth_token']);
		$ch->setToken($this->channel->id, 'access_secret', $request_token['oauth_token_secret']);

		return $url;
	}

	/**
	 * sendMessage
	 *
	 * @param   string  $message  Param
	 * @param   array   $data     Param
	 *
	 * @return	array
	 */
	public function sendMessage($message, $data)
	{
		$isAuth = $this->isAuth();

		if (!$isAuth)
		{
			return array(
				false,
				JText::_('COM_AUTOTWEET_CHANNEL_TUMBLR_NOT_AUTH_ERR')
			);
		}

		$result = array(false, 'Tumblr Unknown', null);

		try
		{
			$media_mode = $this->getMediaMode();

			// Test / Photo / Link
			$type = $this->channel->params->get('posttype', 'text');
			$basehostname = $posttype = $this->channel->params->get('basehostname');

			if (($media_mode == 'message') && ($type == 'photo'))
			{
				return array(
					false,
					JText::_('COM_AUTOTWEET_CHANNEL_TUMBLR_NOT_MEDIA_ERR')
				);
			}

			$image_url = null;

			if ($media_mode != 'message')
			{
				$image_url = $data->image_url;
			}

			if ((empty($image_url)) && ($type == 'photo'))
			{
				return array(
					false,
					JText::_('COM_AUTOTWEET_CHANNEL_TUMBLR_NOT_MEDIA_ERR')
				);
			}

			$this->getApiInstance();

			$parameters = array();
			$parameters['type'] = $type;

			switch ($type)
			{
				case 'photo':
					$parameters['caption'] = $message;
					$parameters['link'] = $data->org_url;
					$parameters['source'] = $image_url;
					break;
				case 'link':
					$parameters['title'] = $data->title;
					$parameters['url'] = $data->org_url;
					$parameters['description'] = $message;
					break;
				default;
					$parameters['type'] = 'text';
					$parameters['title'] = $data->title;
					$parameters['body'] = $this->renderPost($this->channel->id, 'tumblrchannel', $message, $data);
			}

			$post = $this->tumblrClient->post('blog/' . $basehostname . '/post', $parameters);

			if (($post->meta->status == 200) || ($post->meta->status == 201))
			{
				$result = array(
					true
				);
				$result[] = $post->meta->status . ' - ' .
					(isset($post->response->id) ? 'http://' . $basehostname . '/' . $post->response->id : $post->meta->msg);
			}
			else
			{
				$result = array(
					false
				);
				$result[] = $post->meta->status . ' - ' . $post->meta->msg;
			}
		}
		catch (Exception $e)
		{
			return array(
				false,
				$e->getMessage()
			);
		}

		return $result;
	}

	/**
	 * includeHashTags
	 *
	 * @return  bool
	 */
	public function includeHashTags()
	{
		return $this->channel->params->get('hashtags', true);
	}

	/**
	 * getSocialUrl
	 *
	 * @param   object  $user  Param
	 *
	 * @return	string
	 */
	public function getSocialUrl($user)
	{
		if ((isset($user->blogs)) && (count($user->blogs) > 0))
		{
			return $user->blogs[0]->url;
		}
		else
		{
			return 'https://www.tumblr.com';
		}
	}
}
