<?php
/**
 * @package     Extly.Components
 * @subpackage  com_autotweet - A powerful social content platform to manage multiple social networks.
 *
 * @author      Prieco S.A. <support@extly.com>
 * @copyright   Copyright (C) 2007 - 2015 Prieco, S.A. All rights reserved.
 * @license     http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        http://www.extly.com http://support.extly.com
 */
// No direct access
defined('_JEXEC') or die('Restricted access');

/**
 * VirtualManager
 *
 * @package     Extly.Components
 * @subpackage  com_autotweet
 * @since       1.0
 */
class VirtualManager
{
	const EVERGREEN_TYPE_RANDOM = 1;
	const EVERGREEN_TYPE_RANDOM_HITS = 2;
	const EVERGREEN_TYPE_RANDOM_DATER = 3;
	const EVERGREEN_TYPE_RANDOM_DATEO = 4;
	const EVERGREEN_TYPE_SEQUENCE = 5;

	const LAST_ADVANCED_ATTRS_SEQ = 'last_advanced_attrs_seq';

	/**
	 * getInstance
	 *
	 * @return  object
	 */
	public static function getInstance()
	{
		static $instance = null;

		if (!$instance)
		{
			$instance = new VirtualManager;
		}

		return $instance;
	}

	/**
	 * isWorking
	 *
	 * @param   JDate  $now  Params
	 *
	 * @return  bool
	 */
	public function isWorking($now = null)
	{
		if (!$now)
		{
			$now = JFactory::getDate();
		}

		$extension = $this->_loadExtension();

		if ($extension->xtform->get('works7x24', true))
		{
			return true;
		}

		$dayofweek = $now->format('w');
		$working_days = $extension->xtform->get('working_days', array());

		if (array_search($dayofweek, $working_days) === false)
		{
			// $logger = AutotweetLogger::getInstance();
			// $logger->log(JLog::INFO, 'VirtualManager not working dayofweek ' . $dayofweek, $working_days);

			// Not working today
			return false;
		}

		// $logger = AutotweetLogger::getInstance();

		$now_utc = $now->toSql();
		list($now_date, $now_time) = EParameter::getDateTimeParts($now_utc);

		// $logger->log(JLog::INFO, 'now UTC ' . $now_utc);
		$now_time = strtotime($now_utc);

		// $logger->log(JLog::INFO, 'now_time timestamp ' . $now_time);

		$start_time = $now_date . ' ' . $extension->xtform->get('start_time');

		// $logger->log(JLog::INFO, 'start_time ' . $start_time);

		$start_time = strtotime($start_time);

		// $logger->log(JLog::INFO, 'start_time timestamp ' . $start_time);

		$end_time = $now_date . ' ' . $extension->xtform->get('end_time');

		// $logger->log(JLog::INFO, 'end_time ' . $end_time);

		$end_time = strtotime($end_time);

		// $logger->log(JLog::INFO, 'end_time timestamp ' . $end_time);

		// Fix for 24hs overflow
		if ($end_time < $start_time)
		{
			$end_time += (24 * 3600);

			// $logger->log(JLog::INFO, 'FIXED end_time ' . JFactory::getDate($end_time)->toSql());
			// $logger->log(JLog::INFO, 'FIXED end_time timestamp ' . $end_time);
		}

		$result = (($now_time >= $start_time) && ($now_time <= $end_time));

		/*
		if (!$result)
		{
			$logger = AutotweetLogger::getInstance();
			$msg = 'VirtualManager not working (' .
				$start_time . ' <= ' .
				$now_time . ' <= ' .
				$end_time . ')';
			$logger->log(JLog::INFO, $msg);
		}
		*/

		return $result;
	}

	/**
	 * getPublishUpDates
	 *
	 * @param   int  $max  Param
	 *
	 * @return	string
	 */
	public function getPublishUpDates($max = 30)
	{
		$dates = array();
		$extension = $this->_loadExtension();
		$evergreen_freq_mhdmd = $extension->xtform->get('evergreen_freq_mhdmd');

		if ((!$evergreen_freq_mhdmd) || (empty($evergreen_freq_mhdmd)))
		{
			return $dates;
		}

		$advancedattrs = F0FModel::getTmpInstance('Advancedattrs', 'AutoTweetModel');
		$advancedattrs->setState('evergreentype_id', PlgAutotweetBase::POSTTHIS_YES);
		$total = $advancedattrs->getTotal();

		if ($total == 0)
		{
			return $dates;
		}

		$current_date = JFactory::getDate();
		$dpcheck = EParameter::getComponentParam(CAUTOTWEETNG, 'dpcheck_time_intval', 12) * 3600;
		$current_unix = $current_date->toUnix() + $dpcheck;
		$datePublishUp = new DateTime;

		for ($i = 0; ($i < $max); $i++)
		{
			$datePublishUp->setTimestamp($current_unix);
			$next = TextUtil::nextScheduledDate($evergreen_freq_mhdmd, $datePublishUp);

			$dates[] = $next;
			$current_date = JFactory::getDate($next);
			$current_unix = $current_date->toUnix();
		}

		return $dates;
	}

	/**
	 * enqueueEvergreenMessage
	 *
	 * @param   int  $max  Params
	 *
	 * @return  bool
	 */
	public function enqueueEvergreenMessage($max)
	{
		$extension = $this->_loadExtension();
		$evergreen_type = $extension->xtform->get('evergreen_type', 1);
		$channel_restriction = $extension->xtform->get('channelchooser');
		$evergreen_freq_mhdmd = $extension->xtform->get('evergreen_freq_mhdmd');

		if ((!$evergreen_freq_mhdmd) || (empty($evergreen_freq_mhdmd)))
		{
			return false;
		}

		$next = TextUtil::nextScheduledDate($evergreen_freq_mhdmd);

		$automators = F0FModel::getTmpInstance('Automators', 'AutoTweetModel');

		if (!$automators->lastRunCheck('virtualmanager', 0, $next))
		{
			return false;
		}

		$advancedattrs = F0FModel::getTmpInstance('Advancedattrs', 'AutoTweetModel');
		$advancedattrs->setState('evergreentype_id', PlgAutotweetBase::POSTTHIS_YES);
		$n = $advancedattrs->getTotal();

		// $logger = AutotweetLogger::getInstance();
		// $logger->log(JLog::INFO, 'enqueueEvergreenMessage: n=' . $n);

		if ($n == 0)
		{
			return false;
		}

		$result = false;

		for ($i = 0; ($i < $max); $i++)
		{
			if ($evergreen_type == self::EVERGREEN_TYPE_RANDOM)
			{
				$this->_enqueueEvergreenRandom($n, $channel_restriction);
				$result = true;
			}

			if ($evergreen_type == self::EVERGREEN_TYPE_SEQUENCE)
			{
				$this->_enqueueEvergreenSequence($channel_restriction);
				$result = true;
			}
		}

		return $result;
	}

	/**
	 * _enqueueEvergreenRandom
	 *
	 * @param   int    $n                    Params
	 * @param   array  $channel_restriction  Params
	 *
	 * @return  void
	 */
	private function _enqueueEvergreenRandom($n, $channel_restriction)
	{
		$i = rand(1, $n);

		$advancedattrs = F0FModel::getTmpInstance('Advancedattrs', 'AutoTweetModel');
		$advancedattrs->setState('evergreentype_id', PlgAutotweetBase::POSTTHIS_YES);
		$advancedattrs->setState('limitstart', $i - 1);
		$advancedattrs->setState('limit', 1);

		$result = $advancedattrs->getList();

		if (!empty($result))
		{
			$advancedattrs = $result[0];
			$this->_activateEvergreen($advancedattrs->request_id);
			$this->_applyChannelRestriction($advancedattrs, $channel_restriction);
		}
	}

	/**
	 * _enqueueEvergreenSequence
	 *
	 * @param   array  $channel_restriction  Params
	 *
	 * @return  void
	 */
	private function _enqueueEvergreenSequence($channel_restriction)
	{
		$i = $this->_getLastAdvancedattrsSeq();

		$advancedattrs = F0FModel::getTmpInstance('Advancedattrs', 'AutoTweetModel');
		$advancedattrs->setState('evergreentype_id', PlgAutotweetBase::POSTTHIS_YES);
		$advancedattrs->setState('nextseq', $i);
		$advancedattrs->setState('limit', 1);
		$result = $advancedattrs->getList();

		if (!empty($result))
		{
			$advancedattrs = $result[0];
			$this->_activateEvergreen($advancedattrs->request_id);
			$this->_setLastAdvancedattrsSeq($advancedattrs->id);
			$this->_applyChannelRestriction($advancedattrs, $channel_restriction);
		}
		else
		{
			// Wrapped, just continue from the beginning
			$advancedattrs->setState('nextseq', 0);
			$result = $advancedattrs->getList();

			if (!empty($result))
			{
				$advancedattrs = $result[0];
				$this->_activateEvergreen($advancedattrs->request_id);
				$this->_setLastAdvancedattrsSeq($advancedattrs->id);
				$this->_applyChannelRestriction($advancedattrs, $channel_restriction);
			}
		}
	}

	/**
	 * _activateEvergreen
	 *
	 * @param   int  $request_id  Param
	 *
	 * @return  void
	 */
	private function _activateEvergreen($request_id)
	{
		// $logger = AutotweetLogger::getInstance();
		// $logger->log(JLog::INFO, '_activateEvergreen: request_id=' . $request_id);

		$request = F0FModel::getTmpInstance('Requests', 'AutoTweetModel')->getTable();
		$now = JFactory::getDate();

		$request->load($request_id);

		if (!empty($channel_restriction))
		{
			$channels = $channel_restriction;
		}

		$request->save(
			array(
				'published' => 0,
				'publish_up' => $now->toSql()
			)
		);
	}

	/*
	 Just in case Priority
	$n = count($evergreens);

	$n = $n * $n;
	$i = rand(1, $n);
	$i = round(sqrt($arg)) + 1;
	*/

	/**
	 * _loadExtension
	 *
	 * @return  Table
	 */
	private function &_loadExtension()
	{
		static $extension = null;

		if (!$extension)
		{
			$manager = EExtensionHelper::getExtensionId('system', 'autotweetautomator');

			// Load the model
			$extensions = F0FModel::getTmpInstance('Extensions', 'ExtlyModel');
			$extension = $extensions->getItem($manager);
		}

		return $extension;
	}

	/**
	 * _getLastAdvancedattrsSeq
	 *
	 * @return	int
	 */
	private function _getLastAdvancedattrsSeq()
	{
		$extension = $this->_loadExtension();

		return $extension->xtform->get(self::LAST_ADVANCED_ATTRS_SEQ, 0);
	}

	/**
	 * _setLastAdvancedattrsSeq
	 *
	 * @param   int  $advancedattrs_id  Param
	 *
	 * @return	string
	 */
	private function _setLastAdvancedattrsSeq($advancedattrs_id)
	{
		$extension = $this->_loadExtension();
		$extension->xtform->set(self::LAST_ADVANCED_ATTRS_SEQ, $advancedattrs_id);

		$allData = $extension->getData();
		$allData['xtform'] = $extension->xtform;

		$extensions = F0FModel::getTmpInstance('Extensions', 'ExtlyModel');
		$extensions->save($allData);
	}

	/**
	 * _setLastAdvancedattrsSeq
	 *
	 * @param   object  $data                 Param
	 * @param   array   $channel_restriction  Params
	 *
	 * @return	string
	 */
	private function _applyChannelRestriction($data, $channel_restriction)
	{
		$data->xtform = EForm::paramsToRegistry($data);
		$data->xtform->set('channels', $channel_restriction);
		$data->xtform->set('channels_text', TextUtil::createChannelsText($channel_restriction));

		$advancedattrs = F0FModel::getTmpInstance('Advancedattrs', 'AutoTweetModel');
		$advancedattrs->save($data);
	}
}
