<?php

/**
 * @package     Extly.Components
 * @subpackage  com_autotweet - A powerful social content platform to manage multiple social networks.
 *
 * @author      Prieco S.A. <support@extly.com>
 * @copyright   Copyright (C) 2007 - 2015 Prieco, S.A. All rights reserved.
 * @license     http://http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        http://www.extly.com http://support.extly.com
 */
// No direct access
defined('_JEXEC') or die('Restricted access');

/**
 * AutotweetModelManagers
 *
 * @package     Extly.Components
 * @subpackage  com_autotweet
 * @since       1.0
 */
class AutotweetModelManagers extends ExtlyModelExtensions
{
	/**
	 * Public class constructor
	 *
	 * @param   array  $config  The configuration array
	 */
	public function __construct($config = array())
	{
		$config['name'] = 'Extensions';
		$config['table'] = 'Extension';

		parent::__construct($config);
	}

	/**
	 * Method to get a table object, load it if necessary.
	 *
	 * @param   string  $name     The table name. Optional.
	 * @param   string  $prefix   The class prefix. Optional.
	 * @param   array   $options  Configuration array for model. Optional.
	 *
	 * @throws Exception
	 *
	 * @return  F0FTable  A F0FTable object
	 */
	public function getTable($name = '', $prefix = null, $options = array())
	{
		return parent::getTable($this->table, 'ExtlyTable', $options);
	}

	/**
	 * This method runs before the $data is saved to the $table. Return false to
	 * stop saving.
	 *
	 * @param   array   &$data   Param
	 * @param   JTable  &$table  Param
	 *
	 * @return bool
	 */
	protected function onBeforeSave(&$data, &$table)
	{
		if (!TextUtil::isValidCronjobExpr($data['xtform']['evergreen_freq_mhdmd']))
		{
			$this->setError(JText::_('COM_AUTOTWEET_MANAGERS_ERR_CRONEXPR'));

			return false;
		}

		$result = parent::onBeforeSave($data, $table);

		$params = json_decode($data['params']);

		$params->start_time = EParameter::convertLocalUTC($params->start_time);
		$params->start_time = EParameter::getTimePart($params->start_time);

		$params->end_time = EParameter::convertLocalUTC($params->end_time);
		$params->end_time = EParameter::getTimePart($params->end_time);

		$data['params'] = json_encode($params);

		return $result;
	}
}
