<?php

/**
 * @package     Extly.Components
 * @subpackage  com_autotweet - A powerful social content platform to manage multiple social networks.
 *
 * @author      Prieco S.A. <support@extly.com>
 * @copyright   Copyright (C) 2007 - 2015 Prieco, S.A. All rights reserved.
 * @license     http://http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        http://www.extly.com http://support.extly.com
 */
// No direct access
defined('_JEXEC') or die('Restricted access');

F0FModel::getAnInstance('Channels', 'AutoTweetModel')->getTable();

/**
 * AutotweetModelUserChannels
 *
 * @package     Extly.Components
 * @subpackage  com_autotweet
 * @since       1.0
 */
class AutotweetModelUserChannels extends AutotweetModelChannels
{
	private $fbTokenInfo = null;

	private $fbUser = null;

	/**
	 * Sets the list of IDs from the request data
	 *
	 * @param   array  $data  Params
	 *
	 * @return F0FModel
	 */
	public function setIDsFromData($data)
	{
		$safeHtmlFilter = JFilterInput::getInstance();

		$cid = 0;
		$id = 0;
		$kid = 0;

		// Get the ID or list of IDs from the request or the configuration
		if (array_key_exists('cid', $data))
		{
			$cid = $safeHtmlFilter->clean($data['cid'], 'ARRAY');
		}

		if (array_key_exists('id', $data))
		{
			$id = $safeHtmlFilter->clean($data['id'], 'INT');
		}

		$keyName = $this->getTable($this->table)->getKeyName();

		if (array_key_exists($keyName, $data))
		{
			$id = $safeHtmlFilter->clean($data[$keyName], 'INT');
		}

		if (is_array($cid) && !empty($cid))
		{
			$this->setIds($cid);
		}
		else
		{
			if (empty($id))
			{
				$this->setId($kid);
			}
			else
			{
				$this->setId($id);
			}
		}

		return $this;
	}

	/**
	 * loadParentChannel
	 *
	 * @param   int  $channelTypeId  Params
	 *
	 * @return F0FTable
	 */
	public function loadParentChannel($channelTypeId)
	{
		if (!$channelTypeId)
		{
			throw new Exception('Channel Type is empty');
		}

		$userChannels = F0FModel::getTmpInstance('UserChannels', 'AutoTweetModel');
		$userChannels->setState('frontendchannel', 1);
		$userChannels->setState('channeltype', $channelTypeId);
		$channel = $userChannels->getFirstItem();

		if (!$channel)
		{
			throw new Exception('Channel not found');
		}

		return $channel;
	}

	/**
	 * getDataFromParent
	 *
	 * @param   F0FTable  &$parentChannel  Parent channel
	 *
	 * @return array
	 */
	public function getDataFromParent(&$parentChannel)
	{
		$username = ucfirst(JFactory::getUser()->username);

		$channel = $parentChannel->getData();
		$xtform = EForm::paramsToRegistry($parentChannel);
		unset($channel['params']);

		$now = JFactory::getDate()->toSql();
		$now = EParameter::convertUTCLocal($now);

		// Cloning parent channel
		$channel['id'] = 0;
		$channel['name'] = JText::sprintf('COM_AUTOTWEET_USERCHANNELS_CHANNEL_NAME', $username, $channel['description']);
		$channel['description'] = JText::sprintf('COM_AUTOTWEET_USERCHANNELS_CHANNEL_DESC', $username, $now);
		$channel['scope'] = 'U';
		$channel['xtform'] = $xtform->toArray();
		$channel['xtform']['frontendchannel'] = 0;

		unset($channel['created']);
		unset($channel['created_by']);
		unset($channel['modified']);
		unset($channel['modified_by']);
		unset($channel['checked_out']);
		unset($channel['checked_out_time']);
		unset($channel['ordering']);
		unset($channel['error_message']);

		// Facebook
		unset($channel['access_token']);
		unset($channel['user_id']);
		unset($channel['issued_at']);
		unset($channel['expires_at']);
		unset($channel['fbchannel_id']);
		unset($channel['fbchannel_access_token']);
		unset($channel['channel_issued_at']);
		unset($channel['channel_expires_at']);

		// Twitter
		unset($channel['access_token']);
		unset($channel['access_token_secret']);
		unset($channel['user_id']);

		// LinkedIn
		unset($channel['oauth_user_token']);
		unset($channel['oauth_user_secret']);
		unset($channel['user_id']);

		// LinkedIn OAuth2
		unset($channel['access_token']);
		unset($channel['user_id']);

		return $channel;
	}

	/**
	 * getDataFromChannel
	 *
	 * @param   F0FTable  $channel  Channel
	 *
	 * @return array
	 */
	public function getDataFromChannel($channel)
	{
		$xtform = $channel->xtform;
		$data = $channel->getData();
		$data['xtform'] = $xtform->toArray();

		return $data;
	}

	/**
	 * getDataFromAuth
	 *
	 * @param   int    $channelTypeId  Params
	 * @param   array  &$channel       New channel
	 * @param   array  &$data          Auth data
	 *
	 * @return void
	 */
	/**
	 * getDataFromAuth
	 *
	 * @param   int    $channelTypeId  Params
	 * @param   array  &$channel       New channel
	 * @param   array  &$data          Auth data
	 *
	 * @return void
	 */
	public function getDataFromAuth($channelTypeId, &$channel, &$data)
	{
		F0FModel::getTmpInstance('ChannelTypes', 'AutoTweetModel')->getItem($channelTypeId);

		if (($channelTypeId == AutotweetModelChanneltypes::TYPE_FB_CHANNEL)
			|| ($channelTypeId == AutotweetModelChanneltypes::TYPE_FBLINK_CHANNEL))
		{
			$app_id = $channel['xtform']['app_id'];
			$app_secret = $channel['xtform']['secret'];
			$access_token = $this->_getFbExtend($app_id, $app_secret, $data['access_token']);

			$channel['xtform']['access_token'] = $access_token;
			$channel['xtform']['user_id'] = $this->fbUser['id'];
			$channel['xtform']['issued_at'] = $this->fbTokenInfo['issued_at'];
			$channel['xtform']['expires_at'] = $this->fbTokenInfo['expires_at'];

			$fbAppHelper = new FbAppHelper($app_id, $app_secret, $access_token);

			if (!$fbAppHelper->login())
			{
				throw new Exception('Facebook Login Failed!');
			}

			$userChannel = $fbAppHelper->getUserChannel();

			$channel['xtform']['fbchannel_id'] = $userChannel['id'];
			$fbchannel_access_token = ($userChannel['access_token'] ? $userChannel['access_token'] : $access_token);
			$channel['xtform']['fbchannel_access_token'] = $fbchannel_access_token;

			$tokenInfo = $fbAppHelper->getDebugToken($fbchannel_access_token);

			if ( (isset($tokenInfo['data']['is_valid'])) && (!$tokenInfo['data']['is_valid']) )
			{
				$message = $tokenInfo['data']['error']['code'] . ' - ' . $tokenInfo['data']['error']['message'];

				throw new Exception($message);
			}

			$channel['xtform']['channel_issued_at'] = $tokenInfo['issued_at'];
			$channel['xtform']['channel_expires_at'] = $tokenInfo['expires_at'];
		}

		if ($channelTypeId == AutotweetModelChanneltypes::TYPE_TW_CHANNEL)
		{
			$channel['xtform']['access_token'] = $data['access_token'];
			$channel['xtform']['access_token_secret'] = $data['access_token_secret'];
			$channel['xtform']['user_id'] = $data['user_id'];
			$channel['xtform']['social_url'] = $data['social_url'];
		}

		if (($channelTypeId == AutotweetModelChanneltypes::TYPE_LINK_CHANNEL)
			|| ($channelTypeId == AutotweetModelChanneltypes::TYPE_LIGROUP_CHANNEL))
		{
			$channel['xtform']['oauth_user_token'] = $data['oauth_user_token'];
			$channel['xtform']['oauth_user_secret'] = $data['oauth_user_secret'];
			$channel['xtform']['user_id'] = $data['id'];
		}

		if ($channelTypeId == AutotweetModelChanneltypes::TYPE_LIOAUTH2_CHANNEL)
		{
			$channel['xtform']['access_token'] = $data['access_token'];
			$channel['xtform']['expires_in'] = $data['expires_in'];
			$channel['xtform']['user_id'] = $data['user_id'];
			$channel['xtform']['social_url'] = $data['social_url'];
			$channel['xtform']['expires_date'] = $data['expires_date'];
		}
	}

	/**
	 * _getFbExtend
	 *
	 * @param   int     $app_id        Params
	 * @param   string  $secret        Param
	 * @param   string  $access_token  Param
	 *
	 * @return string
	 */
	private function _getFbExtend($app_id, $secret, $access_token)
	{
		$fbAppHelper = new FbAppHelper($app_id, $secret, $access_token);

		if (!$fbAppHelper->login())
		{
			throw new Exception('Facebook Login Failed!');
		}

		$extended_token = $fbAppHelper->getExtendedAccessToken();

		if (!$extended_token)
		{
			throw new Exception(
				JText::sprintf('COM_AUTOTWEET_UNABLETO', 'extend token')
			);
		}

		$this->fbTokenInfo = $fbAppHelper->getDebugToken($extended_token);
		$this->fbUser = $fbAppHelper->getUser();

		return $extended_token;
	}

	/**
	 * getUserAuthParams
	 *
	 * @param   int  $channelTypeId  Params
	 * @param   int  $channelId      Params
	 *
	 * @return array
	 */
	public function getUserAuthParams($channelTypeId, $channelId = null)
	{
		$params = array(
				'channeltype_id' => $channelTypeId
			);

		if (($channelTypeId == AutotweetModelChanneltypes::TYPE_FB_CHANNEL)
			|| ($channelTypeId == AutotweetModelChanneltypes::TYPE_FBLINK_CHANNEL))
		{
			$channel = $this->loadParentChannel($channelTypeId);
			$xtform = EForm::paramsToRegistry($channel);
			$params['app_id'] = $xtform->get('app_id');
		}

		if ($channelTypeId == AutotweetModelChanneltypes::TYPE_TW_CHANNEL)
		{
			$session = JFactory::getSession();
			$session->set('channelTypeId', $channelTypeId);
			$session->set('channelId', $channelId);

			$channel = $this->loadParentChannel($channelTypeId);
			$xtform = EForm::paramsToRegistry($channel);

			$consumer_key = $xtform->get('consumer_key');
			$consumer_secret = $xtform->get('consumer_secret');
			$appHelper = new TwAppHelper($consumer_key, $consumer_secret);
			$appHelper->login();

			$url = $appHelper->getRequestTokenUrl();

			if (!$url)
			{
				throw new Exception('Invalid getRequestTokenUrl');
			}

			$params['request_token_url'] = $url;
		}

		if (($channelTypeId == AutotweetModelChanneltypes::TYPE_LINK_CHANNEL)
			|| ($channelTypeId == AutotweetModelChanneltypes::TYPE_LIGROUP_CHANNEL))
		{
			$session = JFactory::getSession();
			$session->set('channelTypeId', $channelTypeId);
			$session->set('channelId', $channelId);

			$channel = $this->loadParentChannel($channelTypeId);
			$xtform = EForm::paramsToRegistry($channel);

			$api_key = $xtform->get('api_key');
			$secret_key = $xtform->get('secret_key');
			$appHelper = new LiAppHelper($api_key, $secret_key);
			$appHelper->login();

			$url = $appHelper->getRequestTokenUrl();

			if (!$url)
			{
				throw new Exception('Invalid getRequestTokenUrl');
			}

			$params['request_token_url'] = $url;
		}

		if ($channelTypeId == AutotweetModelChanneltypes::TYPE_LIOAUTH2_CHANNEL)
		{
			$session = JFactory::getSession();
			$session->set('channelTypeId', $channelTypeId);
			$session->set('channelId', $channelId);

			$channel = $this->loadParentChannel($channelTypeId);
			$appHelper = new LiOAuth2ChannelHelper($channel);
			$url = $appHelper->getAuthorizationUrl();

			if (!$url)
			{
				throw new Exception('Invalid getRequestTokenUrl');
			}

			$params['request_token_url'] = $url;
		}

		return $params;
	}
}
