<?php
/**
 * @version     2.1.0
 * @package     com_cloudbackup
 * @copyright   Copyright (C) 2012 - 2013. Bhartiy Web Technologies. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 * @author      Izhar Aazmi <info@bhartiy.com> - http://bhartiy.com
 *
 * CloudBackup is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 2
 * as published by the Free Software Foundation.
 *
 * CloudBackup is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CloudBackup; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 * or see http://www.gnu.org/licenses/.
**/

// No direct access
defined('_JEXEC') or die;

/**
 * Cloudbackup helper.
 */
class CloudbackupHelper
{
	/**
	 * Configure the Linkbar.
	 */
	public static function addSubmenu($vName = '')
	{
		JSubMenuHelper::addEntry(
			JText::_('COM_CLOUDBACKUP_TITLE_BACKUPS'),
			'index.php?option=com_cloudbackup&view=backups',
			$vName == 'backups'
		);
		JSubMenuHelper::addEntry(
			JText::_('COM_CLOUDBACKUP_TITLE_PROFILES'),
			'index.php?option=com_cloudbackup&view=profiles',
			$vName == 'profiles'
		);

	}

	/**
	 * Gets a list of the actions that can be performed.
	 *
	 * @return	JObject
	 * @since	1.6
	 */
	public static function getActions()
	{
		$user	= JFactory::getUser();
		$result	= new JObject;

		$assetName = 'com_cloudbackup';

		$actions = array(
			'core.admin', 'core.manage', 'core.create', 'core.edit', 'core.edit.own', 'core.edit.state', 'core.delete'
		);

		foreach ($actions as $action) {
			$result->set($action, $user->authorise($action, $assetName));
		}

		return $result;
	}
	
	public static function getRefreshToken($clientkeys)
	{
		if(!function_exists('curl_init')) 
		{
			JError::raiseWarning(505, JText::_('COM_CLOUDBACKUP_CURL_NOT_AVAILABLE'));
			return false;
		}
		
		$redirect_uri  = JUri::getInstance()->toString(array('scheme', 'host', 'port'));
		$redirect_uri .= JRoute::_('index.php?option=com_cloudbackup&task=config.receivetoken', false);
		$url  = 'https://accounts.google.com/o/oauth2/token';
		$fields = array(
				'code'		=> $clientkeys['authcode'],
				'client_id' 	=> $clientkeys['client_id'],
				'client_secret' => $clientkeys['client_secret'],
				'redirect_uri' 	=> urlencode($redirect_uri),
				'grant_type'	=> 'authorization_code'
			);
		
		$fields_string = array();
		foreach ($fields as $key => $value)	$fields_string[] = "{$key}={$value}" ;
		$fields_string = implode( '&', $fields_string);
		
		$ch = curl_init();
		curl_setopt($ch, CURLOPT_URL, $url);
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
		curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($ch, CURLOPT_POST, count($fields));
		curl_setopt($ch, CURLOPT_POSTFIELDS, $fields_string);
		
		echo $return   = curl_exec($ch);
		$response = curl_getinfo($ch, CURLINFO_HTTP_CODE);
		curl_close($ch);

		$return = json_decode($return);
		if($response == 200) {
			return $return->refresh_token;
		} elseif($response == 400) {
			JError::raiseWarning(505, JText::sprintf('COM_CLOUDBACKUP_AUTH_ERROR', $return->error));
			return false;
		} else {
			JError::raiseWarning(505, JText::_('COM_CLOUDBACKUP_AUTH_ERROR_UNKNOWN'));
			return false;
		}		
	}

	public static function getDriveService()
	{
		static $service = null;
		if(empty($service)) 
		{
			// This may be accessed by external entity, don't trust global constants
			$com    = dirname(dirname( __FILE__ ));
			JTable::addIncludePath($com.DS.'tables');
			$table  = JTable::getInstance('Config', 'CloudbackupTable');
			$table->load(1);
			$tokens = $table->getProperties();
			
			if(empty($tokens['refresh_token'])) {
				JError::raiseWarning(505, JText::_('COM_CLOUDBACKUP_NOT_CONFIGURED_GDRIVE'));
				return false;
			}
			
			require_once $com.'/assets/google/apiClient.php';
			require_once $com.'/assets/google/contrib/apiDriveService.php';
			$tokenb = array('id'=>null, 'client_id'=>null, 'client_secret'=>null, 'refresh_token'=>null, 'access_token'=>null);
			$tokens = array_merge($tokenb, $tokens);
			$client = new apiClient();
			$client->setClientId($tokens['client_id']);
			$client->setClientSecret($tokens['client_secret']);
			$client->refreshToken($tokens['refresh_token']);
			$client->setRedirectUri('urn:ietf:wg:oauth:2.0:oob');
			$service = new apiDriveService($client);
			
			if(empty($service)) {
				JError::raiseWarning(504, JText::_('COM_CLOUDBACKUP_FAILED_LOADSERVICE_GDRIVE'));
				return false;
			}
		}
		return $service;
	}
	
}
