<?php
/**
 * @version     2.1.0
 * @package     com_cloudbackup
 * @copyright   Copyright (C) 2012 - 2013. Bhartiy Web Technologies. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 * @author      Izhar Aazmi <info@bhartiy.com> - http://bhartiy.com
 *
 * CloudBackup is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 2
 * as published by the Free Software Foundation.
 *
 * CloudBackup is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CloudBackup; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 * or see http://www.gnu.org/licenses/.
**/

defined('_JEXEC') or die;

jimport('joomla.application.component.modellist');

/**
 * Methods supporting a list of Cloudbackup records.
 */
class CloudbackupModelbackups extends JModelList
{

    /**
     * Constructor.
     *
     * @param    array    An optional associative array of configuration settings.
     * @see        JController
     * @since    1.6
     */
    public function __construct($config = array())
    {
        if (empty($config['filter_fields'])) {
            $config['filter_fields'] = array(
				'id', 'f.id',
                'filename', 'f.filename',
                'dateadded', 'f.dateadded',
                'filesize', 'f.filesize',
                'mime', 'f.mime',
            );
        }

        parent::__construct($config);
    }

	/**
	 * Method to auto-populate the model state.
	 *
	 * Note. Calling getState in this method will result in recursion.
	 */
	protected function populateState($ordering = null, $direction = null)
	{
		// Initialise variables.
		$app = JFactory::getApplication('administrator');

		// Load the filter state.
		$search= $app->getUserStateFromRequest($this->context.'.filter.search', 'filter_search');
		$this->setState('filter.search', $search);

		$date1 = $app->getUserStateFromRequest($this->context.'.filter.date1', 'filter_date1');
		$date1 = (date('Y-m-d', strtotime($date1)) == $date1) ? $date1 : null;
		$this->setState('filter.date1', $date1);
		
		$date2 = $app->getUserStateFromRequest($this->context.'.filter.date2', 'filter_date2');
		$date2 = (date('Y-m-d', strtotime($date2)) == $date2) ? $date2 : null;
		$this->setState('filter.date2', $date2);
		
		$type  = $app->getUserStateFromRequest($this->context.'.filter.type', 'filter_type');
		$this->setState('filter.type', $type);
		
		$published = $app->getUserStateFromRequest($this->context.'.filter.state', 'filter_published', '', 'string');
		$this->setState('filter.state', $published);
		
		// Load the parameters.
		$params = JComponentHelper::getParams('com_cloudbackup');
		$this->setState('params', $params);
		
		// List state information.
		parent::populateState('f.dateadded', 'desc');
	}

	/**
	 * Method to get a store id based on model configuration state.
	 *
	 * This is necessary because the model is used by the component and
	 * different modules that might need different sets of data or different
	 * ordering requirements.
	 *
	 * @param	string		$id	A prefix for the store id.
	 * @return	string		A store id.
	 * @since	1.6
	 */
	protected function getStoreId($id = '')
	{
		// Compile the store id.
		$id.= ':' . $this->getState('filter.search');
		$id.= ':' . $this->getState('filter.state');

		return parent::getStoreId($id);
	}

	/**
	 * Build an SQL query to load the list data.
	 *
	 * @return	JDatabaseQuery
	 * @since	1.6
	 */
	protected function getListQuery()
	{
		// Create a new query object.
		$db		= $this->getDbo();
		$query	= $db->getQuery(true);

		// Select the required fields from the table.
		$query->select('f.`id`, f.`filename`, f.`filesize`, f.`mime`, f.`dateadded`')
				->from('`#__cloudbackup_files` AS f');
		
		// Filter by search in title
		$search = $this->getState('filter.search');
		if (!empty($search)) {
			if (stripos($search, 'id:') === 0) {
				$query->where('f.id = '.(int) substr($search, 3));
			} else {
				$search = $db->Quote('%'.$db->getEscaped($search, true).'%');
				$query->where('( f.filename LIKE '.$search.' )');
			}
		}
		
		$offset = strtotime(JHTml::date('2012-08-20', 'Y-m-d H:i:s')) - strtotime('2012-08-20');

		$date1 = $this->getState('filter.date1');
		if (!empty($date1)) {
			$date1 = date('Y-m-d H:i:s', strtotime($date1) - $offset);
			$query->where('( f.dateadded >= '.$db->quote($date1).' )');
		}
        
		$date2 = $this->getState('filter.date2');
		if (!empty($date2)) {
			$date2 = date('Y-m-d H:i:s', strtotime($date2. ' 23:59:59') - $offset);
			$query->where('( f.dateadded <= '.$db->quote($date2).' )');
		}
        
		$type = $this->getState('filter.type');
		if (!empty($type)) {
			$type = $db->Quote('%/site-'.$db->getEscaped($type, true).'-%');
			$query->where('( f.filename LIKE '.$type.' )');	// jugaaad.izharaazmi.com
		}
        
		// Add the list ordering clause.
        $orderCol	= $this->state->get('list.ordering');
        $orderDirn	= $this->state->get('list.direction');
        if ($orderCol && $orderDirn) {
            $query->order($db->escape($orderCol.' '.$orderDirn));
        }

		return $query;
		
	}
	
	public function getItems()
	{
		$items = parent::getItems();
		if(!empty($items))
		{
			$db    = JFactory::getDbo();
			$query = $db->getQuery(true);
			$query->select('`id`, `cloud`, `cloudid`, `userid`, `dateadded`')
					->from($db->qn('#__cloudbackup_backups'));
			
			foreach($items as $i => $item)
			{
				$query->clear('where')
						->where($db->qn('fileid') . ' = ' . $db->quote($item->id) );
				$db->setQuery($query);
				$backups = $db->loadObjectList();
				if($db->getErrorNum()) {
					JError::raiseWarning('503', JText::sprintf('COM_CLOUDBACKUP_BACKUP_LIST_ERROR', $db->getErrorMsg()));
				}
				$items[$i]->backups = $backups;
			}
		}
		return $items;
	}
}
