<?php
/**
 * @version     2.1.0
 * @package     com_cloudbackup
 * @copyright   Copyright (C) 2012 - 2013. Bhartiy Web Technologies. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 * @author      Izhar Aazmi <info@bhartiy.com> - http://bhartiy.com
 *
 * CloudBackup is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 2
 * as published by the Free Software Foundation.
 *
 * CloudBackup is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CloudBackup; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 * or see http://www.gnu.org/licenses/.
**/

// No direct access
defined('_JEXEC') or die;

jimport('joomla.application.component.view');

/**
 * View class for a list of Cloudbackup.
 */
class CloudbackupViewBackups extends JViewLegacy
{
	protected $items;
	protected $pagination;
	protected $state;

	/**
	 * Display the view
	 */
	public function display($tpl = null)
	{
		$this->state		= $this->get('State');
		$this->items		= $this->get('Items');
		$this->pagination	= $this->get('Pagination');

		// Check for errors.
		if (count($errors = $this->get('Errors'))) {
			JError::raiseError(500, implode("\n", $errors));
			return false;
		}

		$this->addToolbar();
		parent::display($tpl);
	}

	/**
	 * Add the page title and toolbar.
	 *
	 * @since	1.6
	 */
	protected function addToolbar()
	{
		require_once JPATH_COMPONENT.'/helpers/cloudbackup.php';

		$state	= $this->get('State');
		$canDo	= CloudbackupHelper::getActions($state->get('filter.category_id'));

		JToolBarHelper::title(JText::_('COM_CLOUDBACKUP_TITLE_BACKUPS'), 'backups.png');
		if ($canDo->get('core.admin')) {
			    JToolBarHelper::divider();
			    JToolBarHelper::custom('backups.configgdrive', 'gdrive.png', 'gdrive.png','COM_CLOUDBACKUP_TOOLBAR_CONFIG_GDRIVE', false);
			    // JToolBarHelper::custom('backups.configdropbox', 'dropbox.png', 'dropbox.png', 'COM_CLOUDBACKUP_TOOLBAR_CONFIG_DROPBOX', false);
        }
		// Delete action is future prospect
		
		$doc = JFactory::getDocument();
		$doc->addStyleDeclaration('.icon-32-gdrive { background:url('.JUri::base().'/components/com_cloudbackup/assets/images/gdrive-32.png)}');
		// $doc->addStyleDeclaration('.icon-32-dropbox { background:url('.JUri::base().'/components/com_cloudbackup/assets/images/dropbox.png)}');

		if ($canDo->get('core.admin')) {
			JToolBarHelper::preferences('com_cloudbackup');
		}
	}

	protected function readableFilesize($bytes)
	{
		switch(true)
		{
			case $bytes > 1000000000:
				$size = round($bytes / 1000000000, 3) . ' GB'; break;
			case $bytes > 1000000:
				$size = round($bytes / 1000000, 3) . ' MB'; break;
			case $bytes > 1000:
				$size = round($bytes / 1000, 3) . ' KB'; break;
			case $bytes > 1:
				$size = round($bytes, 3) . ' KB'; break;
			default:
				$size = round($bytes, 3) . ' B';
		}
		return $size;
	}
	
}