/**
 * @package Extly.Components
 * @subpackage com_autotweet - AutoTweet posts content to social channels
 *             (Twitter, Facebook, LinkedIn, etc).
 *
 * @author Prieco S.A.
 * @copyright Copyright (C) 2007 - 2015 Prieco, S.A. All rights reserved.
 * @license http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link http://www.extly.com http://support.extly.com
 */

/* jslint plusplus: true, browser: true, sloppy: true */
/* global jQuery, Request, Joomla, alert, Backbone */

var	ChannelView = Backbone.View.extend({
		tagName:  'tr',
		template: _.template(channelTemplate),
		events : {
			'click a.publish' : 'onPublish',
			'click a.unpublish' : 'onUnpublish'
		},
		initialize: function () {
		},
		render: function () {
			var html = this.template(this.model.toJSON());
			this.$el.html(html);
			this.$el.addClass('enabled-channel success');

			return this;
		},

		onPublish : function(e) {
			var view = this;

			this.lastEvent = e;

			// Step 1 - Re-Authorize
			this.attributes.eventsHub.trigger("userChannel:re-authorize", {
				channelId : view.attributes.channelId,
				channelView : this
			});

			e.preventDefault();
			return false;
		},

		onReAuthorized : function() {
			var publishAction = new PublishAction();
			return this.executeAction(this.lastEvent.currentTarget, publishAction);
		},

		onUnpublish : function(e) {
			var unpublishAction = new UnpublishAction();

			e.preventDefault();
			return this.executeAction(e.currentTarget, unpublishAction);
		},

		executeAction : function(target, theAction) {
			var view = this;

			view.attributes.spinner.addClass('loading');
			UserChannelHelper.showError(view, null);

			theAction.fetch({
				data : {
					_token : view.attributes.token,
					channelId : view.attributes.channelId,
					authParams : UserChannelHelper.getAuthParams()
				},

				wait : true,
				dataType : 'text',
				success : function(message, resp, options) {
					view.attributes.spinner.removeClass('loading');

					if (message.get('status')) {
						view.model = message;
						view.render();
					} else {
						UserChannelHelper.showError(view, message);
					}
				},
				error : function(model, fail, xhr) {
					view.attributes.spinner.removeClass('loading');
					UserChannelHelper.showError(view, fail.responseText);
				}
			});

			return false;
		}
});

// 'click a.authorize-enabled' : 'onAuthorize',

/*
onAuthorize : function(e) {
	var authorizeAction = new AuthorizeAction();

	e.preventDefault();
	return this.executeAction(e.currentTarget, authorizeAction);
},
*/