<?php
/**
 * @version     2.1.0
 * @package     com_cloudbackup
 * @subpackage  plg_cloudbackup_gdrive
 * @copyright   Copyright (C) 2012 - 2013. Bhartiy Web Technologies. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 * @author      Izhar Aazmi <info@bhartiy.com> - http://bhartiy.com
 *
 * CloudBackup is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 2
 * as published by the Free Software Foundation.
 *
 * CloudBackup is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CloudBackup; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 * or see http://www.gnu.org/licenses/.
**/
	
// no direct access
defined('_JEXEC') or die;

// Cloud backup component helper is needed, shall be isolated within this plugin only in next version
require_once JPATH_ADMINISTRATOR.DS.'components/com_cloudbackup/helpers/cloudbackup.php';

/**
* CloudBackup Cloud Service Plugin
*
* @package		Joomla.Plugin
* @subpackage	System.CloudBackup
*/
class  plgCloudbackupGDrive extends JPlugin
{
	protected $_errors = null;
	private $_folder   = null;
	
	public function onCloudUpload($uploadables, $folder, &$errors)
	{
		$this->_folder = $this->getFolderId($folder);
		$this->_errors = &$errors;
		
		$uploaded = array();
		foreach($uploadables as $i => $uploadable)
		{
			if(!empty($this->_folder)) 
			{
				$uploaded[] = $this->upload2Drive($uploadable);
			}
		}
		return $uploaded;
	}
	
	private function upload2Drive($uploadable)
	{
		$service  = CloudbackupHelper::getDriveService();
		if(empty($service)) {
			$this->_errors[] = 'Error: Failed to create reference to the Google Drive service API.';
			return false;
		}
		
		$file   = new DriveFile();
		$file->setTitle(basename($uploadable['filename']));
		$file->setDescription($uploadable['comment']);
		$file->setMimeType($uploadable['mime']);
		$parent = new ParentReference();
		$parent->setId($this->_folder);
		$file->setParents(array($parent));
		
		try { 
			$data = file_get_contents($uploadable['filename']);
			$createdFile = $service->files->insert($file, array('data' => $data, 'mimeType' => $uploadable['mime']));
		} catch (Exception $e) {
			JError::raiseWarning(504, JText::_('PLG_CLOUDBACKUP_GDRIVE_UPLOAD_ERROR', $e->getMessage()));
			$this->_errors[] = 'Error: Failed to Upload file to Google Drive - '.$uploadable['filename'].' Error :'.$e->getMessage();
		}
		
		if(!empty($createdFile['id'])) {
			$this->_errors[] = 'Message: File Uploaded to Google Drive - '.$uploadable['comment'];
			return array(
				'fileid'   => $uploadable['id'],
				'cloud'    => 'gdrive',
				'cloudid'  => $createdFile['id'], 
				'userid'   => $this->getUserInfo(),
				'dateadded'=> JFactory::getDate()->toSql(),
			);
		}
		
		return false;
	}
	
	private function getFolderId($foldername)
	{
		if(empty($foldername))
		{
			$siteurl    = str_replace('/', '_', JUri::getInstance()->toString(array('host', 'port')) . JUri::root(true));
			$foldername = 'CloudBackups of '. $siteurl;
		}
		$service    = CloudbackupHelper::getDriveService();
		
		$mimeType   = 'application/vnd.google-apps.folder';
		$parameters = array('q' => "title = '{$foldername}' and mimeType = '{$mimeType}' and trashed = false", 'fields' => 'items/id');
		$directory  = $service->files->listFiles($parameters);
		
		if(!empty($directory['items'][0]['id'])) 
		{
			return $directory['items'][0]['id'];
		}
		else 
		{
			// Create Folder on Google Drive
			$file = new DriveFile();
			$file->setTitle($foldername);
			$file->setDescription('Backups created by:  CloudBackup for Joomla, powered by: Bhartiy Web Technologies (Bhartiy.com)');
			$file->setMimeType($mimeType);
			$parent  = new ParentReference();
			$parent->setId('root');
			$file->setParents(array($parent));
			$directory = $service->files->insert($file);
			if(!empty($directory['id'])) return $directory['id'];
		}
		
		$this->_errors[] = 'Error: Failed to get folder on Google Drive for upload target. Selected folder name was '. $foldername;
		return false;
	}

	private function getUserInfo()
	{
		static $userid = null;
		if($userid) return $userid;
		
		$app    = JFactory::getApplication();
		
		$db     = JFactory::getDbo();
		$query  = $db->getQuery(true);
		$query->select($db->qn('refresh_token'))
				->select($db->qn('client_id'))
				->select($db->qn('client_secret'))
				->from($db->qn('#__cloudbackup_config'))
				->where($db->qn('cloud_alias') .' = '. $db->quote('gdrive'));
		$db->setQuery($query);
		$api	= $db->loadObject();
		
		$url 	= 'https://accounts.google.com/o/oauth2/token';
		$header = array('Content-Type' => 'application/x-www-form-urlencoded');
		$data   = "client_id={$api->client_id}".
				  "&client_secret={$api->client_secret}".
				  "&refresh_token={$api->refresh_token}".
				  '&grant_type=refresh_token';
		$opts   = array(
					CURLOPT_RETURNTRANSFER  => true,
					CURLOPT_FAILONERROR 	=> false,  
					CURLOPT_SSL_VERIFYPEER  => false,
				);

		try 
		{
			$ch = curl_init($url);
			curl_setopt_array($ch, $opts);
			curl_setopt($ch, CURLOPT_HTTPHEADER, $header);
			curl_setopt($ch, CURLOPT_POSTFIELDS, $data);
			$result = curl_exec($ch);
			curl_close($ch);
		}
		catch(Exception $e)
		{
			return false;
		}

		$result = json_decode($result);
		if(empty($result->access_token)) {
			return false;
		}
		
		$url    = 'https://www.googleapis.com/oauth2/v1/userinfo?access_token='.urlencode($result->access_token);
		try 
		{
			$ch = curl_init($url);
			curl_setopt_array($ch, $opts);
			$result = curl_exec($ch);
			curl_close($ch);
		}
		catch(Exception $e)
		{
			return false;
		}
		
		$result = json_decode($result);
		if(empty($result->email)) {
			return false;
		}
		return $userid = $result->email;
	}

}