<?php

/**
 * @package     Extly.Components
 * @subpackage  PlgContentAutotweetTwitterCard - Plugin AutoTweet NG TwitterCard Tags-Extension for Joomla!
 *
 * @author      Prieco S.A. <support@extly.com>
 * @copyright   Copyright (C) 2007 - 2015 Prieco, S.A. All rights reserved.
 * @license     http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        http://www.extly.com http://support.extly.com
 */
// No direct access
defined('_JEXEC') or die('Restricted access');

/**
 * Opengraph class.
 *
 * @package     Extly.Plugins
 * @subpackage  autotweettwittercard
 * @since       1.0
 */
class TwittercardHelper
{
	protected $title;

	protected $type;

	protected $description;

	protected $imgSrc;

	protected $author;

	protected static $OG_TITLE = 'twitter:title';

	protected static $OG_TYPE = 'twitter:card';

	protected static $OG_DESC = 'twitter:description';

	protected static $OG_AUTHOR = 'twitter:creator';

	protected static $OG_IMAGE = 'twitter:image:src';

	/**
	 * __construct - Class for inserting OG-tags in the HTML header
	 *
	 * @param   string  $title   Param
	 * @param   string  $type    Param
	 * @param   string  $desc    Param
	 * @param   string  $img     Param
	 * @param   string  $author  Param
	 */
	public function __construct($title='', $type='', $desc='', $img='', $author='')
	{
		$this->title = $title;
		$this->type = $type;
		$this->description = $desc;
		$this->imgSrc = $img;
		$this->author = $author;
	}

	/**
	 * Method for creating and inserting meta tags in the html header
	 *
	 * @param   string  $property  Param
	 * @param   string  $content   Param
	 *
	 * @return	void
	 */
	protected function insertTag($property, $content)
	{
		$document = JFactory::getDocument();
		$doctype    = $document->getType();

		if ($doctype !== 'html' || $content == '')
		{
			return;
		}

		$sanitizedContent = htmlentities(strip_tags($content), ENT_QUOTES, "UTF-8");
		$meta = '<meta property="' . $property . '" content="' . $sanitizedContent . '" />';
		$document->addCustomTag($meta);
	}

	/**
	 * Method for inserting tags in the HTML header
	 *
	 * @return	void
	 */
	public function insertTags()
	{
		$this->insertTag(self::$OG_TITLE, $this->title);
		$this->insertTag(self::$OG_TYPE, $this->type);
		$this->insertTag(self::$OG_DESC, $this->description);
		$this->insertTag(self::$OG_IMAGE, $this->imgSrc);
		$this->insertTag(self::$OG_AUTHOR, $this->author);
	}

	/**
	 * __set
	 *
	 * @param   string  $name   Param
	 * @param   string  $value  Param
	 *
	 * @return	void
	 */
	public function __set($name, $value)
	{
		$this->$name = $value;
	}

	/**
	 * __get
	 *
	 * @param   string  $name  Param
	 *
	 * @return	mixed
	 */
	public function __get($name)
	{
		return $this->$name;
	}
}
