<?php
/**
 * @package		Header Tags
 * @subpackage	plg_headertags
 * @copyright	Copyright (C) 2013 Conflate. All rights reserved.
 * @license		GNU General Public License <http://www.gnu.org/copyleft/gpl.html>
 * @link		http://www.joomla-specialist.net
 */

defined('_JEXEC') or die('Restricted access');

jimport('joomla.plugin.plugin');

class plgSystemHeaderTags extends JPlugin{

	protected $enabled = true;

	function plgSystemHeaderTags( &$subject, $params ){
		parent::__construct( $subject, $params );
		$this->_plugin = JPluginHelper::getPlugin( 'system', 'headertags' );
		$this->_params = new JRegistry( $this->_plugin->params );
	}

	public function onAfterDispatch(){
		$app = JFactory::getApplication();
		
		if ( $app->isAdmin() || !$this->enabled ) {
			return;
		}
		
		$menu = $app->getMenu();
		$active = $menu->getActive();
		
		//check if the plugin should be disabled for selected pages
		if(($mode = $this->_params->get('en_page_exclude'))){
			$itemid = ($active && isset($active->query['Itemid'])?$active->query['Itemid']:$app->input->getInt('Itemid', 0));
			if($itemid > 0){
				$pages = $this->_params->get('sel_pages', array());
				if($mode == 'exclude' && !in_array($itemid, $pages)){
					$this->enabled = false;
					return;
				}elseif($mode != 'exclude' && in_array($itemid, $pages)){
					$this->enabled = false;
					return;
				}
			}
		}
		
		//check if the plugin should be disabled for selected components
		if(($mode = $this->_params->get('en_component_exclude'))){
			if($active && isset($active->query['option'])){
				$components = $this->_params->get('sel_components', array());
				if($mode == 'exclude' && !in_array($active->query['option'], $components)){
					$this->enabled = false;
					return;
				}elseif($mode != 'exclude' && in_array($active->query['option'], $components)){
					$this->enabled = false;
					return;
				}
			}
		}
		
		if($this->_params->get('content_area') == 'component'){
			$document = JFactory::getDocument();
			$buffer = $document->getBuffer();
			
			if($document->getType() == 'html' && isset($buffer['component'][''])){
				if(JVERSION >= 3){
					$component_output = $buffer['component'][''][''];
				}else{
					$component_output = $buffer['component'][''];
				}
				$tags = $this->_findHeaderTags($component_output);
				if($tags && is_array($tags)){
					$this->_replaceHeaderTags($tags, $component_output);
					$document->setBuffer($component_output, 'component', null);
				}
			}
		}
	}
	
	public function onAfterRender(){
		$app = JFactory::getApplication();
		if ( $app->isAdmin() || !$this->enabled ) {
			return;
		}
		
		if($this->_params->get('content_area') == 'page'){
			$body = JResponse::getBody();
			$tags = $this->_findHeaderTags($body);
			if($tags && is_array($tags)){
				$this->_replaceHeaderTags($tags, $body);
				JResponse::setBody($body);
			}
			
		}
		
	}
	
	private function _findHeaderTags($content){
		$matches = array();
		$regex = '/\<(h[1-7]{1})(.*?)\>(.*?)<\/h[1-7]{1}\>/is';
		preg_match_all( $regex, $content, $matches, PREG_SET_ORDER );
		if(count($matches)){
			$tags = array();
			for($i=1; $i<7;$i++){
				$ht = 'h'.$i;
				$tags[$ht]['count'] = 0;
				$tags[$ht]['skip'] = $this->_params->get($ht.'_skip', 0);
				$tags[$ht]['mandatory'] = $this->_params->get($ht.'_mandatory');
				$tags[$ht]['mandatory_replace'] = $this->_params->get($ht.'_mandatory_replace');
				$tags[$ht]['limit'] = $this->_params->get($ht.'_limit');
				$tags[$ht]['limit_replace'] = $this->_params->get($ht.'_limit_replace');
				$tags[$ht]['exclude_by'] = $this->_params->get($ht.'_exclude_by');
				$tags[$ht]['exclude_value'] = $this->_params->get($ht.'_exclude_value');
				$tags[$ht]['matches'] = array();
			}
			foreach($matches as $match){
				$ht = $match[1];
				if(isset($tags[$ht])){
					//check if this tag is allowed, or that is excluded
					if($tags[$ht]['exclude_value'] != ''){
						if($this->_isExcludedHeaderTag($match, $tags[$ht]['exclude_by'], $tags[$ht]['exclude_value'])) continue;
					}
					$tags[$ht]['count']++;
					$tags[$ht]['matches'][] = $match;
				}
			}
			return $tags;
		}
		return false;
	}
	
	private function _replaceHeaderTags(&$tags, &$content){
		foreach($tags as $ht => $tag){
			//skip the tag if set
			if($tags[$ht]['skip']) continue;
			
			//check if the tag exists and is mandadory
			if($tag['count'] == 0 && $tag['mandatory'] == 1){
				//A mandatory tag, but doesn't occur. Find the first replacement tag en set it to the current tag
				$rht = $tag['mandatory_replace'];
				if($tags[$rht]['count'] > 0){
					$first_match = array_shift($tags[$rht]['matches']);
					
					//replace old match with the new heading tag only once!
					$str_new = '<' . $ht . $first_match[2] . '>' . trim($first_match[3]) . '</' . $ht . '>';
					$content = substr_replace($content, $str_new, strpos($content, $first_match[0]), strlen($first_match[0]));
					
					//update tags array
					$first_match[0] = $str_new;
					$tags[$ht]['matches'][] = $first_match;
					$tags[$ht]['count']++;
					$tags[$rht]['count']--;
					//update $tag
					$tag = $tags[$ht];
				}
			}
			
			//check if the tag has a limit and if it exceeds that limit
			if($tag['limit'] != 'ul' && $tag['count'] > 0 && $tag['limit'] < $tag['count']){
				$over = $tag['count'] - $tag['limit'];
				//The tag has more then the allowed accurences
				$rht = $tag['limit_replace'];
				$exceeded_tags = array_splice($tags[$ht]['matches'], $tag['limit']);
				foreach($exceeded_tags as $match){
					
						//replace old match with the new heading tag only once!
						$str_new = '<' . $rht . $match[2] . '>' . trim($match[3]) . '</' . $rht . '>';
						$content = substr_replace($content, $str_new, strpos($content, $match[0]), strlen($match[0]));
					
						//update tags array
						$match[0] = $str_new;
						if(isset($tags[$rht])){
							$tags[$rht]['matches'] = array_merge(array($match), $tags[$rht]['matches']);
							$tags[$rht]['count']++;
						}
						$tags[$ht]['count']--;

				}
			}
		}
	}
	
	private function _isExcludedHeaderTag(&$tag, $exclude_by, $exclude_value){
		$cleanTag = str_replace($tag[3], '', $tag[0]); //clean the tag from any content so only the tag itself is converted to XML
		$element = new SimpleXMLElement($cleanTag);
		if(isset($element->attributes()->$exclude_by)){
			$attValue = explode(' ', trim($element->attributes()->$exclude_by));
			$testValues = explode("\n", $exclude_value);
			foreach($testValues as $testValue){
				$values = explode(' ', trim($testValue));
				foreach($values as $value){
					$value = trim($value);
					if(!in_array($value, $attValue)) continue 2;
				}
				return true;
			}
		}
		return false;
	}
}
